/*****************************************************************************/
/*                THE FOLLOWING PROGRAM IS THE SOLE PROPERTY OF              */
/*                               RONALD Q. SMITH                             */
/*             CONTAINING HIS PROPRIETARY CONFIDENTIAL INFORMATION           */
/*                       COPYRIGHT RONALD Q. SMITH 1992                      */
/*****************************************************************************/

/* Global data definitions                                                  */

#include <dos.h>
#include <fcntl.h>
#include <sys\types.h>
#include <sys\stat.h>
#include <io.h>
#include <time.h>
#include <clock.h>

/*
PROCEDURE: open_clock

"open_clock" opens a file handle for the CLOCK$ device.

INTERFACE:
        open_clock();

                   open_clock returns the file handle or the value -1 if
                   unable to establish a handle.
*/

extern int open_clock(void)

{
        return(open("CLOCK$",O_RDWR));
}

/*
PROCEDURE: connect

"connect" sets or clears the CONNECTED flag in CLOCK.SYS.  When the CONNECTED 
flag is set, all writes to the clock set both the DOS real time clock and the 
calendar clock.  When the clocks are not CONNECTED, writes only change the
DOS real time clock.

INTERFACE:
        connect(handle, flag);
                handle is the DOS file handle associated with CLOCK$.
                flag is 1 to set CONNECTED and 0 to clear CONNECTED.

                connect returns a value which is 0 if the function was
                performed correctly and non-zero otherwise.
*/

extern int connect(int handle, unsigned char flag)
{
        auto union REGS regs;

/* Send flag to CLOCK$ device                                              */


        regs.x.ax = 0x4403;             /* IOCTL Send Control Data      */
        regs.x.bx = handle;
        regs.x.cx = 1;                  /* Send 1 byte                  */
        regs.x.dx = (int) &flag;        /* Send value in flag           */

        intdos(&regs, &regs);

        return(regs.x.cflag);

}

/*
PROCEDURE: clock_status

"clock_status" gets the current value of the CONNECTED flag and the date and
time from the calendar clock.

INTERFACE:
        clock_status(handle, &connected, &dos_date, &dos_time);

                     handle must be the DOS file handle associated with the
                     CLOCK$ device.

                     connected is set to 0 if the clocks are not CONNECTED
                     and non-zero if they are connected.

                     dos_date and dos_time are structures defined in <dos.h>.
                     The value for day of week is filled in by this procedure.

                     clock_status returns a value of 0 if the operations was
                     performed correctly and non-zero otherwise.
*/

extern int clock_status(int handle, unsigned char *connected,
        struct dosdate_t *dos_date, struct dostime_t *dos_time)

{
        auto union REGS regs;
        auto struct {
            unsigned char connected;
            unsigned int year;
            unsigned char day;
            unsigned char month;
            unsigned char minute;
            unsigned char hour;
            unsigned char hsecond;
            unsigned char second;
            } ioctl_data;

/* Get connected flag and time from CLOCK$ device.                         */


        regs.x.ax = 0x4402;             /* IOCTL Read Control Data      */
        regs.x.bx = handle;
        regs.x.cx = 9;
        regs.x.dx = (int) &ioctl_data;  /* Buffer to get data           */

        intdos(&regs, &regs);

        *connected = ioctl_data.connected;
        dos_time->hsecond = ioctl_data.hsecond;
        dos_time->second = ioctl_data.second;
        dos_time->minute = ioctl_data.minute;
        dos_time->hour = ioctl_data.hour;
        dos_date->day = ioctl_data.day;
        dos_date->month = ioctl_data.month;
        dos_date->year = ioctl_data.year;

        return(regs.x.cflag);
}

/*****************************************************************************/
/*                THE PRECEDING PROGRAM IS THE SOLE PROPERTY OF              */
/*                               RONALD Q. SMITH                             */
/*             CONTAINING HIS PROPRIETARY CONFIDENTIAL INFORMATION           */
/*                       COPYRIGHT RONALD Q. SMITH 1992                      */
/*****************************************************************************/
